/* $Workfile: OtxTone.h $   $Revision: 29 $    $Modtime: 10/01/02 2:11p $  */
#ifndef OTX_TONE_H
#define OTX_TONE_H
/*******************************************************************************

  @doc PublicHwApi OtxTone

  Copyright (C) 1998, 1999 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxTone | OTX Tone Generator/Detector Logical Device Function Prototypes

  @contents2
  OtxTone.h |

  @index func | OtxTone

  @contents2
  OTX Tone Generator/Detector Functions - OtxToneXXX() | Functions available for Tone Generators / Detectors

  @head3 OtxToneXXX() Function Reference |

  @index func | OtxTone

*******************************************************************************/

#include "OtxType.h"
#include "OtxDef.h"

#include "OtxToneD.h"
#include "OtxDataD.h"


#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************

  @func
    Set the a constant value (a byte or a word depending on the data format)
    to be send on a Pipe.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_CONST_VAL_GENERATOR>
    <nl><f OtxToneReadVal>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_RESULT OtxToneConstValGeneratorInit(
  IN OTX_HANDLE hDevice,             // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,     // @parm Format of the Generated Data
  IN OTX_UINT16 nConstWord           // @parm Constant Value to be sent
);



/*******************************************************************************

  @func
    Configure a sampler device that allows to read a sample of an incoming 
    64kbps pipe.

  @comm
    Call <f OtxToneValSamplerReadData> to read the sample.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_VAL_SAMPLER>
    <nl><f OtxToneValSamplerReadData>

*******************************************************************************/
OTX_RESULT OtxToneValSamplerInit(
  IN OTX_HANDLE hDevice,             // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat      // @parm Format of the sample 
);


/*******************************************************************************

  @func
    Read an 8-bit sample from a an incoming 64kbps pipe.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_VAL_SAMPLER>
    <nl><f OtxToneValSamplerInit>

*******************************************************************************/
OTX_RESULT OtxToneValSamplerReadData(
  IN OTX_HANDLE hDevice,             // @parm Handle to the Logical device
  OUT OTX_UINT16 *pSample            // @parm 8-bit sample from the pipe
);



/*******************************************************************************

  @func
    Set the tone effects to be generated by tone generator data filter

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_EFFECTS_GENERATOR>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_RESULT OtxToneEffectsGeneratorInit(
  IN OTX_HANDLE     hDevice,         // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,     // @parm Data Format of the Generated Tone
  IN OtxToneEffectE eToneType,       // @parm Type of tone effect to be generated on the channel
  IN OtxToneGainE   eToneGain        // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Set the custom parameters for each sinewave component to
    be generated by the digital sinewave generator. A multiple 
    frequency sinewave tone be specified by using multiple 
    elements in the <p sqSineWaveData> array. 

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SINEWAVE_GENERATOR>
    <nl><t OtxToneSineWaveDataS>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_RESULT OtxToneSineWaveGeneratorInit(
  IN OTX_HANDLE         hDevice,            // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,            // @parm Data Format of the Generated Sinewave
  IN OtxToneSineWaveDataS sqSineWaveData[], // @parm Array of Sine Wave data (each element specifies one sine wave (frequency and gain))
  IN OTX_INT32 coSineWaveData,              // @parm Number of elements in <p sqSineWaveData> (1-5)
  IN OtxToneGainE       eToneGain           // @parm Digital gain of generated tone
);



/*******************************************************************************

  @func
    Initialize the DTMF generator. The DTMF generator will generate
    one DTMF tone infinitely.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_GENERATOR>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_RESULT OtxToneDtmfGeneratorInit(
  IN OTX_HANDLE         hDevice,     // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,     // @parm Data Format of the Generated DTMF digit
  IN OTX_UINT8          nDigit,      // @parm Digit to be generated
  IN OtxToneGainE       eToneGain    // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Initialize the DTMF sequence generator. The time between tone can be specified
    in the 'nSilence' parameter, and the duration of each tone is determined by the
    valued of the 'nDuration' parameter.

  @comm
    The set the number to be dialled, call function <f OtxToneDtmfDial>

    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DIALER>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_RESULT OtxToneDtmfDiallerInit(
  IN OTX_HANDLE         hDevice,     // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,     // @parm Data Format of the Generated DTMF digits
  IN OTX_INT32          nOnTime,     // @parm Duration of DTMF tones in milliseconds, 12.75 milliseconds resolution (0 for infinity)
  IN OTX_INT32          nOffTime,    // @parm Silence time between of DTMF tones in increments of 12.75ms
  IN OtxToneGainE       eToneGain    // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Set the number to be dialled by the DTMF dialler.

  @comm
    Start dialling by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DIALER>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_RESULT OtxToneDtmfDial(
  IN OTX_HANDLE   hDevice,        // @parm Handle to the Logical device
  IN OTX_UINT8    sqDigits[],     // @parm Digits to send
  IN OTX_INT32    nDigits,        // @parm Number of digits to send
  IN OTX_TASK_REF nTaskRef,       // @parm See <t OTX_TASK_REF>
  IN OTX_TIME     nWaitMaxMs      // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Initialize the Single Frequency sequence generator. The time between tones can
    be specified in the 'nOffTime' parameter, and the duration of each tone is
    determined by the valued of the 'nOntime' parameter.

  @comm
    Start generation by calling the function <f OtxToneFrequencySequencySend>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SEQUENCE_GENERATOR>
    <nl><f OtxToneFrequencySequencySend>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>

*******************************************************************************/
OTX_RESULT OtxToneSequenceGeneratorInit(
  IN OTX_HANDLE         hDevice,     // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat, // @parm Data Format of the Generated DTMF digits
  IN OTX_TIME           nOnTime,     // @parm Length of (tones) in milliseconds
  IN OTX_TIME           nOffTime,    // @parm Delay between (tones) in milliseconds
  IN OtxToneGainE       eToneGain    // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Send a frequency sequence.

  @comm
    Set the tone and pause lengths and the gain by callinf the
    function <f OtxToneSequenceGeneratorInit>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SEQUENCE_GENERATOR>
    <nl><f OtxToneSequenceGeneratorInit>
    <nl><f OtxToneDtmfDial>

*******************************************************************************/
OTX_RESULT OtxToneSequenceSend(
  IN OTX_HANDLE   hDevice,         // @parm Handle to the Logical device
  IN OTX_INT32    sqFreqs[],       // @parm Frequencies to send in Hz
  IN OTX_INT32    nTones,          // @parm Number of Frequencies (tones) to send
  IN OTX_TASK_REF nTaskRef,        // @parm See <t OTX_TASK_REF>
  IN OTX_TIME     nWaitMaxMs       // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Configures the Silence Detector. A silence is detected when a
    certain number of received samples have a combined energy level of
    less than the specified threshold.

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SILENCE_DETECTOR>
    <nl><f OtxToneSilenceDetectorStatus>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneFskDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneSilenceDetectorInit(
  IN OTX_HANDLE hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,  // @parm Data Format of the Monitored Data Stream
  IN OTX_INT32  nSilenceDuration, // @parm Time (in 12.75ms increments) that silence must be present to be reported
  IN OTX_INT32  nThreshold        // @parm Low thresHold stipulates very silent silence for silence to be reported;
                                  // Higher thresHold accepts noise in the silence and silence is still being reported
                                  // Typical value to be used: 4000
);


/*******************************************************************************

  @func
    Reports whether the silence detector has detected silence or a tone

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SILENCE_DETECTOR>
    <nl><f OtxToneSilenceDetectorInit>
    <nl><f OtxToneDtmfDetectorStatus>

*******************************************************************************/
OTX_RESULT OtxToneSilenceDetectorStatus(
  IN  OTX_HANDLE hDevice,      // @parm Handle to the Logical device
  OUT OTX_BOOL  *bSilence      // @parm OTX_TRUE if silence, OTX_FALSE if tone detected
);



/*******************************************************************************

  @func
    Configures the Dial Tone Detector. A dial tone is detected when a
    dial tone has been present for the time period specied with parameter
    nToneDuration.

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DIALTONE_DETECTOR>
    <nl><f OtxToneDialtoneDetectorStatus>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneFskDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneDialtoneDetectorInit(
  IN OTX_HANDLE hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,  // @parm Data Format of the Monitored Data Stream
  IN OTX_INT32  nToneDuration     // @parm Time (in 12.75ms increments) that dialtone must be present to be reported
);


/*******************************************************************************

  @func
    Reports whether the silence detector has detected silence or a tone

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DIALTONE_DETECTOR>
    <nl><f OtxToneDialtoneDetectorInit>
    <nl><f OtxToneDtmfDetectorStatus>
    <nl><f OtxToneSilenceDetectorStatus>

*******************************************************************************/
OTX_RESULT OtxToneDialtoneDetectorStatus(
  IN  OTX_HANDLE hDevice,      // @parm Handle to the Logical device
  OUT OTX_BOOL  *bDialtone     // @parm OTX_TRUE if Dial Tone present, OTX_FALSE if not
);


/*******************************************************************************

  @func
    Configures the DTMF Detector.

  @comm
    The received digits can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored internally
    in which case only the most recently received digit can be retrieved with
    a call to function <f OtxToneDtmfDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DETECTOR>
    <nl><f OtxToneDtmfDetectorStatus>
    <nl><f OtxToneSilenceDetectorInit>
    <nl><f OtxToneDialtoneDetectorInit>
    <nl><f OtxToneFskDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneDtmfDetectorInit(
  IN OTX_HANDLE hDevice,         // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat, // @parm Data Format of the Monitored Data Stream
  IN OTX_BOOL   bStoreInQueue    // @parm If OTX_TRUE, generate events and store the received digits
                                 // in the Event QueusFifo, If OTX_FALSE Don't store. Provide polling test
                                 // with function <f OtxToneDtmfDetectorStatus>
);



/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received DTMF tone. If a tone was received, the function returns OTX_S_OK and
    the last digit in the cDtmfDigit parameter. If no tone was detected, the
    function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DETECTOR>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneSilenceDetectorStatus>

*******************************************************************************/
OTX_RESULT OtxToneDtmfDetectorStatus(
  IN  OTX_HANDLE hDevice,      // @parm Handle to the Logical device
  OUT char      *cDtmfDigit    // @parm Most recent DTMF digit detected (or \0 if none)
);



/*******************************************************************************

  @func
    Initialize the MF sequence generator. The time between tone can be specified
    in the 'nSilence' parameter, and the duration of each tone is determined by the
    valued of the 'nDuration' parameter.

  @comm
    The set the number to be dialled, call function <f OtxToneMfDial>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DIALER>
    <nl><f OtxToneMfGeneratorInit>
    <nl><f OtxToneMfDial>

*******************************************************************************/
OTX_RESULT OtxToneMfDiallerInit(
  IN OTX_HANDLE         hDevice,     // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,     // @parm Data Format of the Generated DTMF digits
  IN OtxMfToneE eMfToneType,         // @parm Type of MF Tone to detect (see <t OtxMfToneE>)
  IN OTX_INT32          nOnTime,     // @parm Duration of MF tones in milliseconds, 12.75 milliseconds resolution (0 for infinity)
  IN OTX_INT32          nOffTime,    // @parm Silence time between of MF tones in milliseconds, 12.75 milliseconds resolution
  IN OtxToneGainE       eToneGain    // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Dial a string of MF digits (tones).

  @comm
    This function is a non-blocking function. The event 
    <e OtxMfDialerEventCodesE.OTX_MF_DIALER_EC_COMPLETE> is generated when the device
    has completed dialing the specified string of MF digits.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DIALER>
    <nl><f OtxToneMfDiallerInit>

*******************************************************************************/
OTX_RESULT OtxToneMfDial(
  IN OTX_HANDLE   hDevice,        // @parm Handle to the Logical device
  IN OTX_UINT8    sqDigits[],     // @parm Digits/Tones (see <t OtxMfToneE>) to send. The digits 
                                  // can either be specified as a numeric value (0-15), or as 
                                  // characters '0' through '9' and 'a' through 'f' (where 'a' 
                                  // through 'f' represents 10 through 15). Please not that the R1 
                                  // digits are in the range of 0-14, and the R2 digits are in the 
                                  // range of 1-15.
  IN OTX_INT32    nDigits,        // @parm Number of digits/tones to send
  IN OTX_TASK_REF nTaskRef,       // @parm See <t OTX_TASK_REF>
  IN OTX_TIME     nWaitMaxMs      // @parm See <t OTX_TIME>
);




/*******************************************************************************

  @func
    Configures the MF Detector.

  @comm
    The received digits can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored internally
    in which case only the most recently received digit can be retrieved with
    a call to function <f OtxToneMfDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DETECTOR>
    <nl><f OtxToneMfDetectorStatus>

*******************************************************************************/
OTX_RESULT OtxToneMfDetectorInit(
  IN OTX_HANDLE hDevice,           // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,   // @parm Data Format of the Monitored Data Stream
  IN OtxMfToneE eMfToneType,       // @parm Type of MF Tone to detect (see <t OtxMfToneE>)
  IN OTX_BOOL   bStoreInQueue,     // @parm If OTX_TRUE, generate events and store the received digits
                                   // in the Event QueusFifo, If OTX_FALSE Don't store. Provide polling test
                                   // with function <f OtxToneMfDetectorStatus>
  IN OTX_BOOL   bReportPause,      // @parm If OTX_TRUE the detector generates an event when a tone ends
  IN OTX_INT16  nMinDuration       // @parm Minimum duration required to validate a digit
);



/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received MF tone. If a tone was received, the function returns 
    OTX_S_OK and the last digit in the cDigit parameter. If no tone was detected, 
    the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DETECTOR>
    <nl><f OtxToneMfDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneMfDetectorStatus(
  IN  OTX_HANDLE hDevice,      // @parm Handle to the Logical device
  OUT char      *nDigit    // @parm Most recent digit detected (or -1 if none)
);


/*******************************************************************************

  @func
    Configures the Dual Tone Detector.

  @comm
    The dual tone detector can detect a number tone pairs (two-frequency tones). 
    The frequencies to detect are supplied as a parameter to this function. This device
    can handle 6 frequencies (FreqA through FreqF). When a certain dual tone is detected
    an event is generated and the detected dual tone is coded according to the table
    below.

  @ex Dual Tone Coding Matrix |

          .         FreqA	 FreqB  FreqC  FreqD  FreqE	 FreqF 
          FreqA    { -1,    1,      2,     4,     7,    11  },
          FreqB    {  1,   -1,      3,     5,     8,    12  },
          FreqC    {  2,    3,     -1,     6,     9,    13  },
          FreqD    {  4,    5,      6,    -1,    10,    14  },
          FreqE    {  7,    8,      9,    10,    -1,    15  },
          FreqF    { 11,   12,     13,    14,    15,    -1  }

   @comm 

    The Dual Tone detector device is highly configurable to be able to detect 
    various tones of various amplitudes. This is a brief description of 
    how the device works:

    <nl> 
         1. The device gathers 102 samples of tone data (12.75 ms). 

    <nl> 
         2. It then runs an algorithm for each of the 6 frequencies that the 
            device can work with. The algorithm will produce energy numbers for each 
            frequency. If the buffer of 102 samples contains a frequency component 
            which closely matches one of the detection frequencies it will produce 
            a high energy number for that frequency. If there is a mismatch in frequencies 
            then the energy number will be low.

    <nl>
         3. The code now has 6 energy numbers to work with. It will now run these 
            numbers through a series of tests to see if they pass the thresholds (which 
            can be specified through attributes through API functions). The first test 
            performed is the 
            <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY>
            threshold. The two highest energy values have to be higher than this threshold 
            to pass as a detected tone. The default value for this threshold is 900. 
            Lower values will make the detection criteria less stringent.

    <nl>
         4. The next test is the 
            <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST>
            threshold. The code will look at the two highest energy levels and calculate 
            their ratio (MaxEnergy/2ndMaxEnergy*100). The ratio must be less than this 
            threshold to pass as a detected tone. The default value for this threshold is 150 (1.5 times). 
            Higher values will make the detection criteria less stringent.

    <nl>
         5. The next test is the 
            <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD> 
            threshold. The code has already looked at the two highest energy levels. It will 
            now look at the third highest energy level. It will calculate the ration 
            between the 2nd highest and the 3rd highest (2ndMaxEnergy/3rdMaxEnergy*100). The 
            ratio must be greater than this threshold to pass as a detected tone. The 
            default value for this threshold is 150 (1.5 times). Lower values will make the detection criteria less stringent.

    <nl> 
         6. The last check performed is the duration test. The duration paramter is supplied 
            in this init function. The detection has to generate a positive and consistent 
            result for a number of 12.75 ms buffers in order for the code to generate a 
            detection event to the API.


    <nl> If the symptom is that you are not the correct detection events, it might be necessary to 
    adjust the thresholds for the device. To further analyze which threshold that need to be 
    adjusted, the 
    <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_VERBOSE_STATUS> attribute can be 
    set to a value higher than zero, which causes the device to generate 
    <e OtxDualToneDetectorEventCodesE.OTX_DUAL_TONE_DETECTOR_EC_VERBOSE_STATUS> events. 
    Higher values generates more verbose debug information. The 
    m_nParam member of the OTX_DUAL_TONE_DETECTOR_EC_VERBOSE_STATUS event provides the following 
    information:


  @ex Verbose Status Information (verbose level 1) |

      Bit 15-0: Bit 31-16:                            Event occurs when
      ~~~~~~~~~ ~~~~~~~~~                             ~~~~~~~~~~~~~~~~~~
      2         n2ndMaxEnergy                         THRESHOLD_TONE_ENERGY test has failed to meet criteria	
      3         nMaxEnergy/n2ndMaxEnergy*100          THRESHOLD_TWIST test has failed to meet criteria	
      4         n2ndMaxEnergy/n3rdMaxEnergy*100       THRESHOLD_RELATIVE_3RD test has failed to meet criteria	
      5         nDualTone ID of detected dual tone    Duration consistency test failed. Another dual 
      .                                               tone was detected before the end of the required 
      .                                               duration.

  @ex Verbose Status Information (verbose level 5) |

     Bit 15-0: Bit 31-16:                             Event occurs when
     ~~~~~~~~~ ~~~~~~~~~                              ~~~~~~~~~~~~~~~~~~
     20        Energy level of tone index 0           once for every 12.75ms buffer
     21        Energy level of tone index 1           once for every 12.75ms buffer
     22        Energy level of tone index 2           once for every 12.75ms buffer
     23        Energy level of tone index 3           once for every 12.75ms buffer
     24        Energy level of tone index 4           once for every 12.75ms buffer
     25        Energy level of tone index 5           once for every 12.75ms buffer


   @comm 
   
    The detected tones can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored internally
    in which case only the most recently received digit can be retrieved with
    a call to function <f OtxToneDualDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DUAL_DETECTOR>
    <nl><f OtxToneDualDetectorStatus>

*******************************************************************************/
OTX_RESULT OtxToneDualDetectorInit(
  IN OTX_HANDLE hDevice,           // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,   // @parm Data Format of the Monitored Data Stream
  IN OTX_UINT32    sqFreq[],       // @parm An array of up to 6 frequencies to be detected.
  IN OTX_INT32     coFreq,         // @parm Number of elements in <p sqFreq>
  IN OTX_BOOL   bStoreInQueue,     // @parm If OTX_TRUE, generate events and store the received digits
  IN OTX_BOOL   bReportPause,      // @parm If OTX_TRUE the detector generates an event when a tone ends
  IN OTX_INT16  nMinDuration       // @parm Minimum duration required to validate a digit
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received dual tone. If a tone was received, the function returns 
    OTX_S_OK and the last code in the cCode parameter. If no tone was detected, 
    the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DUAL_DETECTOR>
    <nl><f OtxToneDualDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneDualDetectorStatus(
  IN  OTX_HANDLE hDevice,      // @parm Handle to the Logical device
  OUT char      *nCode         // @parm Most recent dual tone code detected (or -1 if none)
);


/*******************************************************************************

  @func
    Configures the Single Tone Detector.

  @comm
    The Single tone detector can detect a Single tone. 
    The frequencies to detect are supplied as a parameter to this function. This device
    can handle 6 frequencies (FreqA through FreqF). When a certain single tone is detected
    an event is generated and the detected single tone is coded according to the table
    below.

  @ex Single Tone Coding Matrix |

          FreqA    {0}
          FreqB    {1}
          FreqC    {2}
          FreqD    {3}
          FreqE    {4}
          FreqF    {5}

  @comm 
   
    The detected tones can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored internally
    in which case only the most recently received digit can be retrieved with
    a call to function <f OtxToneSingleDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

   @comm 

    The Single Tone detector device is highly configurable to be able to detect 
    various tones of various amplitudes. This is a brief description of 
    how the device works:

    <nl>
         1. The device gathers 102 samples of tone data (12.75 ms). 
         
    <nl>
         2. It then runs an algorithm for each of the 6 frequencies that the 
            device can work with. The algorithm will produce energy numbers for each 
            frequency. If the buffer of 102 samples contains a frequency component 
            which closely matches one of the detection frequencies it will produce 
            a high energy number for that frequency. If there is a mismatch in frequencies 
            then the energy number will be low.

    <nl>
         3. The code now has 6 energy numbers to work with. It will now run these 
            numbers through a series of tests to see if they pass the thresholds (which 
            can be specified through attributes through API functions). The first test 
            performed is the 
            <e OtxSingleDetectorAttributesE.OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_TONE_ENERGY>
            threshold. The two highest energy values have to be higher than this threshold 
            to pass as a detected tone. The default value for this threshold is 1000. 
            Lower values will make the detection criteria less stringent.

    <nl>
         4. The next check performed is the duration test. The duration paramter is supplied 
            in this init function. The detection has to generate a positive and consistent 
            result for a number of 12.75 ms buffers in order for the code to generate a 
            detection event to the API.

    <nl>
         5. The last test is the 
            <e OtxSingleDetectorAttributesE.OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_VARIANCE_ENERGY> 
            threshold. The code has already found the highest energy level. This threshold
            determines how much the energy level is allow to vary from one 12.75ms buffer
            to the next. The code will look at the current energy level and compare to the
            average energy level detected so far. The default value for this threshold is 
            200. Higher values will make the detection criteria less stringent.


    <nl> If noticing symptoms of incorrect detection events, it might be necessary to 
    adjust the thresholds for the device. To further analyze which threshold that need to be 
    adjusted, the 
    <e OtxSingleDetectorAttributesE.OTX_ATTR_SINGLE_TONE_DETECTOR_VERBOSE_STATUS> attribute can be 
    set to a value higher than zero, which causes the device to generate 
    <e OtxSingleToneDetectorEventCodesE.OTX_SINGLE_TONE_DETECTOR_EC_VERBOSE_STATUS> events. 
    Higher values generates more verbose debug information. The 
    m_nParam member of the OTX_SINGLE_TONE_DETECTOR_EC_VERBOSE_STATUS event provides the following 
    information:


  @ex Verbose Status Information (verbose level 1) |

      Bit 15-0: Bit 31-16:                     Event occurs when
      ~~~~~~~~~ ~~~~~~~~~                      ~~~~~~~~~~~~~~~~~~
      2         n2ndMaxEnergy                  THRESHOLD_TONE_ENERGY test has failed to meet criteria	
      5         Index of detected dual tone    Duration consistency test failed. Another dual 
      6         nMaxEnergy                     THRESHOLD_VARIANCE test has failed to meet criteria. The
      .                                        energy level varied beyond the threshold (when compared
      .                                        to the average energy level detected so far.

  @ex Verbose Status Information (verbose level 5) |

      Bit 15-0: Bit 31-16:                     Event occurs when
      ~~~~~~~~~ ~~~~~~~~~                      ~~~~~~~~~~~~~~~~~~
      20        Energy level of tone index 0   for every 12.75ms buffer
      21        Energy level of tone index 1   for every 12.75ms buffer
      22        Energy level of tone index 2   for every 12.75ms buffer
      23        Energy level of tone index 3   for every 12.75ms buffer
      24        Energy level of tone index 4   for every 12.75ms buffer
      25        Energy level of tone index 5   for every 12.75ms buffer


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SINGLE_DETECTOR>
    <nl><f OtxToneSingleDetectorStatus>

*******************************************************************************/
OTX_RESULT OtxToneSingleDetectorInit(
  IN OTX_HANDLE hDevice,           // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,   // @parm Data Format of the Monitored Data Stream
  IN OTX_UINT32    sqFreq[],       // @parm An array of up to 6 frequencies to be detected.
  IN OTX_INT32     coFreq,         // @parm Number of elements in <p sqFreq>
  IN OTX_BOOL   bStoreInQueue,     // @parm If OTX_TRUE, generate events and store the received digits
  IN OTX_BOOL   bReportPause,      // @parm If OTX_TRUE the detector generates an event when a tone ends
  IN OTX_INT16  nMinDuration       // @parm Minimum duration required to validate a digit
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received single tone. If a tone was received, the function returns 
    OTX_S_OK and the last code in the cCode parameter. If no tone was detected, 
    the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SINGLE_DETECTOR>
    <nl><f OtxToneSingleDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneSingleDetectorStatus(
  IN  OTX_HANDLE hDevice,      // @parm Handle to the Logical device
  OUT char      *nCode         // @parm Most recent single tone code detected (or -1 if none)
);


/*******************************************************************************

  @func
    Initialize the Logical FSK Detector Device

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_FSK_DETECTOR>
    <nl><f OtxToneSilenceDetectorInit>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneDialtoneDetectorInit>

*******************************************************************************/
OTX_RESULT OtxToneFskDetectorInit(
  IN OTX_HANDLE hDevice,             // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat      // @parm Data Format of the Monitored Data Stream
);


/*******************************************************************************

  @func
    Reads one value (a byte or a word depending on the data format)
    from the specified Channel

*******************************************************************************/
OTX_RESULT OtxToneReadVal(
  IN  OTX_HANDLE hDevice,       // @parm Handle to the Logical device
  OUT OTX_UINT16 *nVal          // @parm Value received on a Pipe
);


#ifdef __cplusplus
}  // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998, 1999


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif  // OTX_TONE_H
